#ifndef HIER_CLUSTERING
#define HIER_CLUSTERING

#include "types.h"

// Dumb implementation of connected (graph) hierarchical clustering
class HierClusteringSlow
{
public:

	HierClusteringSlow() {}

	void init(int numNodes, double *pValues, double *pWeights, int numEdges, double *pEdges);

	int getNumNodes() const {
		return (int)m_nodeValues.size();
	}

	int getNumEdges() const {
		return (int)m_edges.size();
	}

	// computes cluster mean and variance (which we're trying to minimize):
	double clusterMean(const IntSet &nodes);
	double clusterVar(const IntSet &nodes);

	// returns index into m_edges such that the corresponding edge represents the two clusters to merge:
	int findBestMergeEdge(double &bestMergeCost);

	void clustering(int targetNumClusters);

	// assumes 'pRes' is allocated with numNodes elements, writes back cluster index for each node
	// adds +1 for MATLAB indexing:
	void writeBackResult(double *pRes);

	BoolVector m_active; // indicates whether given node is a cluster representative (i.e. node with the smallest index in the cluster)
	IntSetVector m_clusterElements; // for each active cluster c, m_clusterElements[c] is a set of its elements
	IntUPairList m_edges; // each edge indicates a possible join between two active clusters

	DoubleVector m_nodeValues;
	DoubleVector m_nodeWeights;


//	DoubleList m_clusterWeight; // sum of weights of nodes in a given cluster (if active)
//	DoubleList m_clusterValuesSum; // weighted sum of values of cluster nodes (if active)
};

#endif
