#ifndef TUPLES_H
#define TUPLES_H

//#include "types.h"
#include <vector>
#include <assert.h>

template <unsigned int N> class TupleBase
{
public:
	TupleBase() {} // uninitialized

	TupleBase(const int v[N]) { 
		initArr(v);
	}

	void initArr(const int v[N]) {
		for (int i=0; i<N; i++) m_v[i] = v[i];
	}

	inline int getV(int c) const {
		assert(c >= 0 && c < N);
		return m_v[c];
	}

	inline void setV(int c, int val) {
		assert(c >= 0 && c < N);
		m_v[c] = val;
	}

	void sort();
	bool isStrictlyOrdered(); 
	bool isOrdered(); 

	bool operator<(const TupleBase<N> &t) const;
	bool operator==(const TupleBase<N> &t) const;

protected:
	int m_v[N];	
};

template <unsigned int N> class Tuple : public TupleBase<N> { };

template<> class Tuple<2> : public TupleBase<2> {
public:

	Tuple() {}

	Tuple(int v0, int v1) {
		init(v0, v1);
	}

	void init(int v0, int v1) {
		const int tmp[2] = {v0, v1};
		initArr(tmp);
	}

};

template<> class Tuple<3> : public TupleBase<3> {
public:

	Tuple() {}

	Tuple(int v0, int v1, int v2) {
		init(v0, v1, v2);
	}

	void init(int v0, int v1, int v2) {
		const int tmp[3] = {v0, v1, v2};
		initArr(tmp);
	}

};

template<> class Tuple<4> : public TupleBase<4> {
public:

	Tuple() {}

	Tuple(int v0, int v1, int v2, int v3) {
		init(v0, v1, v2, v3);
	}

	void init(int v0, int v1, int v2, int v3) {
		const int tmp[4] = {v0, v1, v2, v3};
		initArr(tmp);
	}

};

template <unsigned int N> class TupleVector : public std::vector<Tuple<N> > { };




#include <algorithm>

template<unsigned int N> void TupleBase<N>::sort()
{
	std::sort(m_v, m_v + N);
}

template<unsigned int N> bool TupleBase<N>::isStrictlyOrdered()
{	
	for (int i=1; i<N; i++)
	{
		if (m_v[i] <= m_v[i-1]) return false;
	}
	return true;
}

template<unsigned int N> bool TupleBase<N>::isOrdered()
{
	for (int i=1; i<N; i++)
	{
		if (m_v[i] < m_v[i-1]) return false;
	}
	return true;
}

template<unsigned int N> bool TupleBase<N>::operator<(const TupleBase<N> &t) const
{
	for (int i=0; i<N; i++)
	{
		if (m_v[i] < t.m_v[i]) return true;
		if (m_v[i] > t.m_v[i]) return false;
		assert(m_v[i] == t.m_v[i]);
	}
	// they're equal
	return false;
}

template<unsigned int N> bool TupleBase<N>::operator==(const TupleBase<N> &t) const
{
	for (int i=0; i<N; i++)
	{		
		if (m_v[i] != t.m_v[i]) return false;	
	}	
	return true;
}



////typedef std::vector<IntUPair> IntUPairList;
//template <unsigned int N> class TupleList : public std::vector<Tuple<N> > {};
//
//template <unsigned int N> class OrderedTuple
//{
//public:
//	OrderedTuple() {} // uninitialized
//
//	OrderedTuple(int v[N]) {
//		for (int i=0; i<N; i++) m_v[i] = v[i];
//		sort();
//	}
//	OrderedTuple(const Tuple<N> &t) {
//		for (int i=0; i<N; i++) m_v[i] = t.getV(i);
//		sort();
//	}
//
//protected:
//
//	int m_v[N];	
//	void sort();
//};

void testTuples();

#endif
