#ifndef POST_GRAPH_CUT_H
#define POST_GRAPH_CUT_H
#include "types.h"
#include <map>
#include <vector>
// POST_GRAPH_CUT Post processing after graph cut to meet constrained facets
//
// Inputs:
//   d  dim, number of dimensions
//   numElements  #T, number of tets
//   pElements  #T by dim+1 list of element indices
//   pValues  #T by 1 list winding number values at each Element
//   numFacets #F, number of facets
//   pFacets  #F by dim list of constrained facet indices
//   pInitialOutside  #T by 1 list of bools whether pElements are considered
//     outside or inside
//   pSmoothnessTerm  #T by #T sparse matrix of smoothness weights
//   DataTermIn  #T list of data weights for in
//   DataTermOut #T list of data weights for out
// Outputs:
//   outside  #T by 1 list of bools whether pElements are considered outside or
//     inside
//
// Note: Everything is of type double to facilitate matlab
void post_graph_cut(
  const int d,
  const int numElements,
  const double * pElements,
  const double * pValues,
  const int numFacets,
  const double * pFacets,
  const std::map<std::pair<int,int>,double> & pSmoothnessTerm,
  const std::vector<double> & pDataTermIn,
  const std::vector<double> & pDataTermOut,
  BoolVector & outside);
#endif
