% Prepare paths
old_warn = warning('OFF','MATLAB:dispatcher:pathWarning');
addpath ../selfintersect/
addpath ../winding_number/
addpath ../post_graph_cut/
addpath matlab_bgl/
addpath maxflow/
warning(old_warn);
%
% Draw a curve
clf;
axis([0 1 0 1]);
axis equal
axis manual;
set(title('Drag to draw a closed curve'),'Visible','on');
V = get_pencil_curve();
% simplify a bit
V = dpsimplify(V,1e-4); 
% facet (edge) list, assuming the user drew a CCW open curve
F = [1:size(V,1)-1;2:size(V,1)]';

% number of subplots
nsp = 4;

set(gcf,'Color',[1 1 1]);
% Do everything twice, once using the input curve as is, next removing a large
% fraction of the input
for r = 1:2
  if r == 2
    % randomly remove some entries
    F = F(randperm(end),:);
    % number of faces to remove
    num_rm = floor(size(F,1)/8);
    F = F(1:end-num_rm,:);
  end

  % Show input
  subplot(2,nsp,(r-1)*nsp+1);
  plot([V(F(:,1),1) V(F(:,2),1)]',[V(F(:,1),2) V(F(:,2),2)]', ...
    '-k','LineWidth',3);
  axis equal;
  axis tight;
  grid off;
  set(gca,'Visible','off');
  if r == 1
    set(title('input'),'Visible','on','FontSize',20);
  else
    set(title(sprintf('after removing %2.0f%% of facets',100*num_rm/size(F,1))), ...
      'Visible','on','FontSize',20);
  end
  % Weird trick to get axis same size despite colorbar
  set(colorbar,'Visible','off');
  drawnow;
  
  % show visualization of w.n. in space around input
  BB = bounding_box(V);
  % Create a (dense) triangle mesh containing this curve
  [DV,DEle] = triangle([V;BB],F,[],'Flags','-c');
  [DV,DEle] = triangle([V;BB],F,[],'Flags','-c', ...
    'MaxArea',1e-4);
  subplot(2,nsp,(r-1)*nsp+2);
  w = winding_number(V,F,barycenter(DV,DEle))/(2*pi);
  if sum(w) < 0
    warning('Flipping sign of w to match convention');
    w = -w;
  end
  trisurf(DEle,DV(:,1),DV(:,2),zeros(size(DV,1),1),w,'EdgeColor','none');
  hold on;
  plot([V(F(:,1),1) V(F(:,2),1)]',[V(F(:,1),2) V(F(:,2),2)]', ...
    '-k','LineWidth',3);
  hold off;
  colormap(jet(255));
  caxis([-1/2 2]);
  set(colorbar,'FontSize',20);
  grid off;
  axis equal;
  axis tight;
  view(2);
  set(gca,'Visible','off');
  set(title('w'),'Visible','on','FontSize',20);
  drawnow;
  
  % Compute a minimal cdt
  subplot(2,nsp,(r-1)*nsp+3);
  [DV,DEle] = cdt(V,F);
  w = winding_number(V,F,barycenter(DV,DEle))/(2*pi);
  if sum(w) < 0
    warning('Flipping sign of w to match convention');
    w = -w;
  end
  trisurf(DEle,DV(:,1),DV(:,2),zeros(size(DV,1),1),w, ...
    'LineWidth',1, ...
    'EdgeColor',[0.5 0.5 0.5]);
  hold on;
  plot([V(F(:,1),1) V(F(:,2),1)]',[V(F(:,1),2) V(F(:,2),2)]', ...
    '-k','LineWidth',3);
  hold off;
  colormap(jet(255));
  caxis([-1/2 2]);
  set(colorbar,'FontSize',20);
  axis equal;
  axis tight;
  grid off;
  view(2);
  set(gca,'Visible','off');
  set(title('cdt'),'Visible','on','FontSize',20);
  drawnow;
  
  % Cut!
  [CEle,cut] = graphcut(DV,DEle,tt(DEle),w);
  [CV,IM] = remove_unreferenced(DV,CEle);
  CEle = IM(CEle);
  CF = outline(CEle);
  subplot(2,nsp,(r-1)*nsp+4);
  trisurf(CEle,CV(:,1),CV(:,2),zeros(size(CV,1),1), ...
    'FaceColor',[0.309804 0.14902 0.027451], ...
    'LineWidth',1, ...
    'EdgeColor',[0.7 0.7 0.7]);
  hold on;
  plot([CV(CF(:,1),1) CV(CF(:,2),1)]',[CV(CF(:,1),2) CV(CF(:,2),2)]', ...
    '-','LineWidth',5,'Color',[0.5 0.5 0.5]);
  plot([V(F(:,1),1) V(F(:,2),1)]',[V(F(:,1),2) V(F(:,2),2)]', ...
    '-k','LineWidth',1.5);
  hold off;
  axis equal;
  axis tight;
  set(colorbar,'Visible','off');
  grid off;
  view(2);
  set(gca,'Visible','off');
  set(title('output mesh'),'Visible','on','FontSize',20);
  drawnow;
end
